assets

package
v0.0.0-...-dfb39b5 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Aug 2, 2026 License: Apache-2.0 Imports: 5 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Files

func Files(log *zap.Logger, aPath string) (files fs.FS)

Files return files from assets path or falls back to embedded files

@todo try to find a way to merge this with auth/assets

Types

This section is empty.

Jump to