Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func TypeRegistry ¶
TypeRegistry is a simplified type registry for the types that AST nodes need to understand.
Types ¶
type ASTNode ¶
type ASTNode struct {
// Meta holds additional node data such as scope references
Meta map[string]any `json:"meta,omitempty"`
Ref string `json:"ref,omitempty"`
Args ASTNodeSet `json:"args,omitempty"`
Symbol string `json:"symbol,omitempty"`
Value *TypedValue `json:"value,omitempty"`
Raw string `json:"raw,omitempty"`
}
func (*ASTNode) CollectSymbols ¶
type ASTNodeSet ¶
type ASTNodeSet []*ASTNode
type TypedValue ¶
type TypedValue struct {
V expr.TypedValue
}
func MakeValueOf ¶
func MakeValueOf(t string, v interface{}) *TypedValue
func WrapValue ¶
func WrapValue(v expr.TypedValue) *TypedValue
func (*TypedValue) MarshalJSON ¶
func (t *TypedValue) MarshalJSON() ([]byte, error)
func (*TypedValue) UnmarshalJSON ¶
func (t *TypedValue) UnmarshalJSON(in []byte) (err error)
Click to show internal directories.
Click to hide internal directories.